package grammar;

import node.*;
import java.io.*;
import java_cup.runtime.*;

%%

%public
%type		Symbol
%char

%{
	private int lineNumber = 1;
	public int lineNumber() { return lineNumber; }
	
	public Symbol token( int tokenType ) {
		System.err.println( "Obtain token " 
			+ sym.terminal_name( tokenType ) + " \"" + yytext() + "\"" );
		return new Symbol( tokenType, yychar, 
			yychar + yytext().length(), yytext() );
		}

%}

charValue	=	\\.|[^\\\"]
stringValue	=	\"{charValue}*\"
intValue	=	[0-9]+
ident		=	[A-Za-z][A-Za-z0-9]*
space		=	[\ \t]
newline		=	\r|\n|\r\n

%%

{newline}	{ lineNumber++; }
{space}		{ }

";"			{ return token( sym.SEMICOLON ); }
","			{ return token( sym.COMMA ); }
"="			{ return token( sym.ASSIGN ); }
"<"			{ return token( sym.LT ); }
"<="		{ return token( sym.LE ); }
"=="		{ return token( sym.EQ ); }
"+"			{ return token( sym.PLUS ); }
"-"			{ return token( sym.MINUS ); }
"*"			{ return token( sym.TIMES ); }
"/"			{ return token( sym.DIVIDE ); }
"("			{ return token( sym.LEFT ); }
")"			{ return token( sym.RIGHT ); }
"{"			{ return token( sym.LEFTCURLY ); }
"}"			{ return token( sym.RIGHTCURLY ); }

"print"		{ return token( sym.PRINT ); }
"while"		{ return token( sym.WHILE ); }
"do"		{ return token( sym.DO ); }

{stringValue}	{ return token( sym.STRINGVALUE ); }
{intValue}	{ return token( sym.INTVALUE ); }
{ident}		{ return token( sym.IDENT ); }

"//"[^\r\n]*	{ }
"/*"~"*/"		{ }

.			{ return token( sym.LEXERROR ); }

<<EOF>>		{ return token( sym.EOF ); }

